﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using FinecarRentCar.DAL;
using FinecarRentCar.Data;
using Microsoft.AspNetCore.Mvc;
using Syncfusion.EJ2.Base;

namespace FinecarRentCar.Controllers.Orders
{
    public class OrdersController : Controller
    {
        private ICarRepository _carRepository = new CarRepository(new FinecarDbContext());
        private FinecarDbContext db = new FinecarDbContext();
        private readonly ApplicationDbContext context;
        public OrdersController(ApplicationDbContext applicationDbContext)
        {
            context = applicationDbContext;
        }

        #region Car Orders
        [Route("administrace/rezervace-vozu")]
        public IActionResult CarOrders()
        {
            return View();
        }
        #endregion

        #region Accessory Orders
        [Route("administrace/rezervace-prislusenstvi")]
        public IActionResult AccessoryOrders()
        {
            return View();
        }
        #endregion

        #region Sales
        [Route("administrace/akce-a-slevy")]
        public IActionResult ActionsAndSales()
        {
            var query = (from carOrder in db.CarOrders
                         group carOrder by new
                         {
                             carOrder.CarID
                         } into o
                         select new
                         {
                             CarId = o.Key.CarID,
                             carCount = o.Count()
                         }).ToList();

            var ids = query.Where(c => c.carCount > 0).Select(c => c.CarId).ToList();

            var carsWithoutOrder = db.Cars.Where(c => !ids.Contains(c.ID)).Select(c => new
            {
                CarFullName = c.TagNumber + " - " + c.Make + " " + c.Model
            }).ToList();

            int countResult = carsWithoutOrder.Count;

            var result = string.Join(",  ", carsWithoutOrder.Select(c => c.CarFullName));
            ViewBag.CarsWithoutOrder = result;
            ViewBag.countResult = countResult;
            return View();
        }
        #endregion

        #region Data & CRUD
        public IActionResult OrdersDataSource([FromBody]DataManagerRequest dm) // OrdersDataSource
        {
            var lister = new ListUsers(context);
            var data = lister.ListUserWithRoles();
            var employeeDataSource = data;

            IEnumerable DataSource = db.RentalOrders.Select(o => new Models.CarOrdersViewModel()
            {
                ID = o.ID,
                DateProcessed = o.DateProcessed,
                CarID = _carRepository.GetCarByOrderId(o.ID).ID,
                Customer = o.Customer,
                Address = o.Address,
                City = o.City,
                ZIPCode = o.ZIPCode,
                Phone = o.Phone,
                Email = o.Email,
                RentStartDate = o.RentStartDate,
                RentEndDate = o.RentEndDate,
                ParkingPlace = o.ParkingPlace,
                Days = (int)((o.RentEndDate - o.RentStartDate)).TotalDays,
                RateApplied = o.RateApplied,
                OrderTotal = o.OrderTotal,
                OrderStatus = o.OrderStatus,
                IsStorno = o.IsStorno,
                IsReservation = o.IsReservation,
                IsRent = o.IsRent,
                IsReturn = o.IsReturn,
                IsLongLease = o.IsLongLease,
                IsRentAction = o.IsRentAction,
                RentActionID = o.RentActionID,
                LimitLongLease = o.LimitLongLease,
                AccessoriesID = o.AccessoriesID,
                Notes = o.Notes,
                StartMilleage = o.StartMilleage,
                EndMilleage = o.EndMilleage,
                TagNumber = _carRepository.GetCarByOrderId(o.ID).TagNumber,
                CarFullName = _carRepository.GetCarByOrderId(o.ID).Make + " " + _carRepository.GetCarByOrderId(o.ID).Model + " " + _carRepository.GetCarByOrderId(o.ID).Performance,
                CarImageFileName = _carRepository.GetCarByOrderId(o.ID).ImageFileName,
                OwnerTaskId = _carRepository.GetEventByOrderId(o.ID).OwnerTaskID,
                OwnerTaskEmail = GetEmplyeeEmailByUserId(_carRepository.GetEventByOrderId(o.ID).OwnerTaskID),
                Updated = o.Updated
            }).ToList();
            DataOperations operation = new DataOperations();
            if (dm.Search != null && dm.Search.Count > 0)
            {
                DataSource = operation.PerformSearching(DataSource, dm.Search);
            }
            if (dm.Sorted != null && dm.Sorted.Count > 0) //Sorting
            {
                DataSource = operation.PerformSorting(DataSource, dm.Sorted);
            }
            if (dm.Where != null && dm.Where.Count > 0) //Filtering
            {
                DataSource = operation.PerformFiltering(DataSource, dm.Where, dm.Where[0].Operator);
            }
            int count = DataSource.Cast<object>().Count();
            if (dm.Skip != 0)
            {
                DataSource = operation.PerformSkip(DataSource, dm.Skip);   //Paging
            }
            if (dm.Take != 0)
            {
                DataSource = operation.PerformTake(DataSource, dm.Take);
            }
            return dm.RequiresCounts ? Json(new { result = DataSource, count = count }) : Json(DataSource);
        }

        public IActionResult RentActionsDataSource([FromBody]DataManagerRequest dm)
        {
            IEnumerable DataSource = db.RentActions.Select(a => new Models.RentActionViewModel()
            {
                ID = a.ID,
                IsActive = a.IsActive,
                IsForRegisteredUser = a.IsForRegisteredUser,
                Sale = Convert.ToDouble(a.Sale) / 100,
                ActionInWorkDays = a.ActionInWorkDays,
                DayRentStatement = a.DayRentStatement,
                ActionInWeek = a.ActionInWeek,
                DayOfWeekRentStatement = a.DayOfWeekRentStatement,
                ValidFrom = a.ValidFrom,
                ValidTo = a.ValidTo,
                CarsInActionId = _carRepository.CarsInAction(a.ID)
            }).ToList();

            DataOperations operation = new DataOperations();
            if (dm.Search != null && dm.Search.Count > 0)
            {
                DataSource = operation.PerformSearching(DataSource, dm.Search);
            }
            if (dm.Sorted != null && dm.Sorted.Count > 0) //Sorting
            {
                DataSource = operation.PerformSorting(DataSource, dm.Sorted);
            }
            if (dm.Where != null && dm.Where.Count > 0) //Filtering
            {
                DataSource = operation.PerformFiltering(DataSource, dm.Where, dm.Where[0].Operator);
            }
            int count = DataSource.Cast<object>().Count();
            if (dm.Skip != 0)
            {
                DataSource = operation.PerformSkip(DataSource, dm.Skip);   //Paging
            }
            if (dm.Take != 0)
            {
                DataSource = operation.PerformTake(DataSource, dm.Take);
            }
            return dm.RequiresCounts ? Json(new { result = DataSource, count = count }) : Json(DataSource);
        }

        public ActionResult AddPartial()
        {
            var carSource = db.Cars.Select(car => new Models.CarsViewModel()
            {
                CarId = car.ID,
                CarFullName = car.TagNumber + " - " + car.Make + " " + car.Model + " " + car.Performance,
                TagNumber = car.TagNumber
            }).ToList();
            ViewBag.carDataSource = carSource;

            var lister = new ListUsers(context);
            var data = lister.ListUserWithRoles();
            ViewBag.employeeDataSource = data.Where(x => x.RoleNames == "Employee");

            var accessoriesSource = db.Accessories.ToList();
            ViewBag.accessoryDataSource = accessoriesSource;

            ViewBag.defaultDate = DateTime.Now;

            List<MonthViewModel> monthData = new List<MonthViewModel>();
            string month = "měsíc";
            string month2_4 = "měsíce";
            string month5_36 = "měsíců";
            int countItem = 37;
            for (int i = 1; i < countItem; i++)
            {
                if (i == 1)
                {
                    monthData.Add(new MonthViewModel { value = i, text = i.ToString() + " " + month });
                }
                else if (i > 1 && i < 5)
                {
                    monthData.Add(new MonthViewModel { value = i, text = i.ToString() + " " + month2_4 });
                }
                else
                {
                    monthData.Add(new MonthViewModel { value = i, text = i.ToString() + " " + month5_36 });
                }
            }

            ViewBag.monthDataSource = monthData;

            return PartialView("_DialogAddPartial");
        }

        public IActionResult EditPartial([FromBody]CRUDModel<Models.CarOrdersViewModel> value)
        {
            var carSource = db.Cars.Select(car => new Models.CarsViewModel()
            {
                CarId = car.ID,
                CarFullName = car.TagNumber + " - " + car.Make + " " + car.Model + " " + car.Performance,
                TagNumber = car.TagNumber
            }).ToList();
            ViewBag.carDataSource = carSource;

            var lister = new ListUsers(context);
            var data = lister.ListUserWithRoles();
            ViewBag.employeeDataSource = data.Where(x => x.RoleNames == "Employee");

            var accessoriesSource = db.Accessories.ToList();
            ViewBag.accessoryDataSource = accessoriesSource;

            ViewBag.defaultDate = DateTime.Now;

            List<MonthViewModel> monthData = new List<MonthViewModel>();
            string month = "měsíc";
            string month2_4 = "měsíce";
            string month5_36 = "měsíců";
            int countItem = 37;
            for (int i = 1; i < countItem; i++)
            {
                if (i == 1)
                {
                    monthData.Add(new MonthViewModel { value = i, text = i.ToString() + " " + month });
                }
                else if (i > 1 && i < 5)
                {
                    monthData.Add(new MonthViewModel { value = i, text = i.ToString() + " " + month2_4 });
                }
                else
                {
                    monthData.Add(new MonthViewModel { value = i, text = i.ToString() + " " + month5_36 });
                }
            }

            ViewBag.monthDataSource = monthData;
            return PartialView("_DialogEditPartial", value.Value);
        }

        public ActionResult ActionAddPartial()
        {
            var carSource = db.Cars.Select(car => new Models.CarsViewModel()
            {
                CarId = car.ID,
                CarFullName = car.TagNumber + " - " + car.Make + " " + car.Model
            }).ToList();
            ViewBag.carDataSource = carSource;

            var query = (from carOrder in db.CarOrders
                         group carOrder by new
                         {
                             carOrder.CarID
                         } into o
                         select new
                         {
                             CarId = o.Key.CarID,
                             carCount = o.Count()
                         }).ToList();

            var ids = query.Where(c => c.carCount > 0).Select(c => c.CarId).ToList();

            var carsWithoutOrder = db.Cars.Where(c=> !ids.Contains(c.ID)).Select(c => new
            {
                CarFullName = c.TagNumber + " - " + c.Make + " " + c.Model
            }).ToList();

            var result = string.Join(",  ", carsWithoutOrder.Select(c=>c.CarFullName));
            ViewBag.CarsWithoutOrder = result;

            return PartialView("_ActionDialogAddPartial");
        }
        public IActionResult ActionEditPartial([FromBody]CRUDModel<Models.RentActionViewModel> value)
        {
            var carSource = db.Cars.Select(car => new Models.CarsViewModel()
            {
                CarId = car.ID,
                CarFullName = car.TagNumber + " - " + car.Make + " " + car.Model,
                TagNumber = car.TagNumber
            }).ToList();
            ViewBag.carDataSource = carSource;

            return PartialView("_ActionDialogEditPartial", value.Value);
        }
        public JsonResult GetCarPrice(int carId, int numberOfDays)
        {
            return Json(new { result = _carRepository.GetPriceCarForDays(carId, numberOfDays) });
        }
        public JsonResult GetCarWeekPrice(int carId)
        {
            return Json(new { result = _carRepository.GetPriceCarForWeek(carId) });
        }

        public JsonResult GetAccessoryPrices(string accessoryIds, int numberOfDays)
        {
            return Json(new { result = _carRepository.GetPriceAccessoriesForDays(accessoryIds, numberOfDays) });
        }
        public JsonResult GetAccessoriesWeekPrice(string accessoryIds)
        {
            return Json(new { result = _carRepository.GetPriceAccessoriesForWeek(accessoryIds) });
        }
        public JsonResult GetCarLongPrice(int carId, int numberOfMonths)
        {
            return Json(new { result = _carRepository.GetLongPriceForCar(carId, numberOfMonths) });
        }
        public JsonResult GetCarNoLimitLongPrice(int carId, int numberOfMonths)
        {
            return Json(new { result = _carRepository.GetLongNoLimitPriceForCar(carId, numberOfMonths) });
        }
        public JsonResult GetCarsInAction(int actionId)
        {
            var carIds = db.CarInActions.Where(a => a.ActionID == actionId).Select(x=>x.CarID).ToList();
            List<Models.Entities.Car> cars = new List<Models.Entities.Car>();
            foreach(var carId in carIds)
            {
                cars.Add(db.Cars.Where(x => x.ID == carId).FirstOrDefault());
            }
            IEnumerable DataSource = cars.Select(c => new Models.CarsViewModel()
            {
                CarId = c.ID,
                CarFullName = c.Make + " " + c.Model + " " + c.Performance,
                TagNumber = c.TagNumber
            }).ToList();

            int count = DataSource.Cast<object>().Count();

            return Json(new { result = DataSource, count = count });
        }
        public JsonResult GetOrdersInAction(int actionId)
        {
            var orderIds = db.OrderInActions.Where(x => x.RentActionID == actionId).Select(x => x.RentOrderID).ToList();
            List<Models.Entities.RentalOrder> orders = new List<Models.Entities.RentalOrder>();
            foreach(var orderId in orderIds)
            {
                orders.Add(db.RentalOrders.Where(x => x.ID == orderId).FirstOrDefault());
            }
            IEnumerable DataSource = orders.Select(o => new Models.CarOrdersViewModel()
            {
                ID = o.ID,
                Customer = o.Customer,
                RentStartDate = o.RentStartDate,
                RentEndDate = o.RentEndDate,
                OrderTotal = o.OrderTotal,
                OwnerTaskEmail = GetEmplyeeEmailByUserId(_carRepository.GetEventByOrderId(o.ID).OwnerTaskID)
            }).ToList();

            int count = DataSource.Cast<object>().Count();

            return Json(new { result = DataSource, coun = count });
        }
        public IActionResult CarDataSource([FromBody]Data dm)
        {
            var DataSource = db.Cars.Select(car => new Models.CarsViewModel()
            {
                CarId = car.ID,
                TagNumber = car.TagNumber,
                Tachometer = car.Tachometer,
                CarFullName = car.Make + " " + car.Model + " " + car.Performance,
                Make = car.Make,
                Model = car.Model,
                Performance = car.Performance,
                ImageFileName = car.ImageFileName,
                CategoryName = _carRepository.GetCategoryNameByCarId(car.ID),
                Meta = car.Meta,
                UrlSlug = car.UrlSeo
            }).ToList();

            var Data = DataSource.ToList();
            int count = DataSource.Count();

            return dm.requiresCounts ? Json(new { result = Data.Skip(dm.skip).Take(dm.take), count = count }) : Json(Data);
        }

        // CRUD
        public JsonResult InsertOrder([FromBody] CRUDModel<Models.CarOrdersViewModel> value)
        {
            if (!value.Value.IsLongLease)
            {
                int prices = 0;
                if (!value.Value.IsRentAction)
                {
                    if ((int)value.Value.RentStartDate.DayOfWeek == 5 && (int)value.Value.RentStartDate.Hour == 16 && (int)value.Value.RentEndDate.DayOfWeek == 1 && (int)value.Value.RentEndDate.Hour == 9)
                    {
                        prices = _carRepository.GetPriceCarForWeek(value.Value.CarID);
                    }
                    else
                    {
                        prices = _carRepository.GetPriceCarForDays(value.Value.CarID, (int)(value.Value.RentEndDate - value.Value.RentStartDate).TotalDays);
                    }
                }
                else
                {

                }

                var entity = new Models.Entities.RentalOrder
                {
                    DateProcessed = DateTime.Now,
                    Customer = value.Value.Customer,
                    Address = value.Value.Address,
                    City = value.Value.City,
                    ZIPCode = value.Value.ZIPCode,
                    Phone = value.Value.Phone,
                    Email = value.Value.Email,
                    RentStartDate = value.Value.RentStartDate,
                    RentEndDate = value.Value.RentEndDate,
                    ParkingPlace = value.Value.ParkingPlace,
                    Days = (int)(value.Value.RentEndDate - value.Value.RentStartDate).TotalDays,
                    OrderTotal = prices,
                    OrderStatus = value.Value.OrderStatus,
                    IsStorno = false,
                    IsReservation = value.Value.IsReservation,
                    IsRent = value.Value.IsRent,
                    IsReturn = false,
                    IsLongLease = false,
                    IsRentAction = value.Value.IsRentAction,
                    RentActionID = value.Value.RentActionID,
                    LimitLongLease = false,
                    AccessoriesID = value.Value.AccessoriesID,
                    Notes = value.Value.Notes,
                    StartMilleage = _carRepository.GetCarById(value.Value.CarID).Tachometer,
                    Updated = DateTime.Now
                };
                db.RentalOrders.Add(entity);
                db.SaveChanges();
            }
            else
            {
                if (value.Value.LimitLongLease)
                {
                    int prices = 0;
                    var months = GetMonthDifference(value.Value.RentStartDate, value.Value.RentEndDate.AddMonths(-13));
                    if (!value.Value.IsRentAction)
                    {
                        prices = _carRepository.GetLongPriceForCar(value.Value.CarID, months);
                    }
                    else
                    {

                    }
                    var entity = new Models.Entities.RentalOrder
                    {
                        DateProcessed = DateTime.Now,
                        Customer = value.Value.Customer,
                        Address = value.Value.Address,
                        City = value.Value.City,
                        ZIPCode = value.Value.ZIPCode,
                        Phone = value.Value.Phone,
                        Email = value.Value.Email,
                        RentStartDate = value.Value.RentStartDate,
                        RentEndDate = value.Value.RentEndDate,
                        ParkingPlace = value.Value.ParkingPlace,
                        Days = (int)(value.Value.RentEndDate - value.Value.RentStartDate).TotalDays,
                        OrderTotal = prices,
                        OrderStatus = value.Value.OrderStatus,
                        IsStorno = false,
                        IsReservation = value.Value.IsReservation,
                        IsRent = value.Value.IsRent,
                        IsReturn = false,
                        IsLongLease = value.Value.IsLongLease,
                        IsRentAction = value.Value.IsRentAction,
                        RentActionID = value.Value.RentActionID,
                        LimitLongLease = true,
                        AccessoriesID = value.Value.AccessoriesID,
                        Notes = value.Value.Notes,
                        StartMilleage = _carRepository.GetCarById(value.Value.CarID).Tachometer,
                        Updated = DateTime.Now
                    };
                    db.RentalOrders.Add(entity);
                    db.SaveChanges();
                }
                else
                {
                    int prices = 0;
                    var months = GetMonthDifference(value.Value.RentStartDate, value.Value.RentEndDate.AddMonths(-13));
                    prices = _carRepository.GetLongNoLimitPriceForCar(value.Value.CarID, months);
                    var entity = new Models.Entities.RentalOrder
                    {
                        DateProcessed = DateTime.Now,
                        Customer = value.Value.Customer,
                        Address = value.Value.Address,
                        City = value.Value.City,
                        ZIPCode = value.Value.ZIPCode,
                        Phone = value.Value.Phone,
                        Email = value.Value.Email,
                        RentStartDate = value.Value.RentStartDate,
                        RentEndDate = value.Value.RentEndDate,
                        ParkingPlace = value.Value.ParkingPlace,
                        Days = (int)(value.Value.RentEndDate - value.Value.RentStartDate).TotalDays,
                        OrderTotal = prices,
                        OrderStatus = value.Value.OrderStatus,
                        IsStorno = false,
                        IsReservation = value.Value.IsReservation,
                        IsRent = value.Value.IsRent,
                        IsReturn = false,
                        IsLongLease = value.Value.IsLongLease,
                        IsRentAction = value.Value.IsRentAction,
                        RentActionID = value.Value.RentActionID,
                        LimitLongLease = false,
                        AccessoriesID = value.Value.AccessoriesID,
                        Notes = value.Value.Notes,
                        StartMilleage = _carRepository.GetCarById(value.Value.CarID).Tachometer,
                        Updated = DateTime.Now
                    };
                    db.RentalOrders.Add(entity);
                    db.SaveChanges();
                }
            }
            int lastOrderId = db.RentalOrders.Select(o => o.ID).Last();

            var carOrder = new Models.Entities.CarOrders
            {
                CarID = value.Value.CarID,
                OrderID = lastOrderId
            };
            db.CarOrders.Add(carOrder);
            db.SaveChanges();

            var orderEvent = new Models.Entities.RentEvent
            {
                StartTime = value.Value.RentStartDate,
                EndTime = value.Value.RentEndDate,
                Subject = _carRepository.GetCarById(value.Value.CarID).TagNumber + " - " + _carRepository.GetCarById(value.Value.CarID).Make + " " + _carRepository.GetCarById(value.Value.CarID).Model,
                Description = "Rezervace vozu " + _carRepository.GetCarById(value.Value.CarID).TagNumber + ", místo přistavení: " + value.Value.ParkingPlace,
                IsAllDay = false,
                OwnerTaskID = value.Value.OwnerTaskId
            };
            db.RentEvents.Add(orderEvent);
            db.SaveChanges();

            int lastEventId = db.RentEvents.Select(e => e.ID).Last();

            var carRentEvent = new Models.Entities.CarOrderRentEvent
            {
                RentalOrderID = lastOrderId,
                RentEventID = lastEventId
            };
            db.CarOrderRentEvents.Add(carRentEvent);
            db.SaveChanges();

            return new JsonResult(value.Value);
        }
        public JsonResult UpdateOrder([FromBody]CRUDModel<Models.CarOrdersViewModel> value)
        {
            if (!value.Value.IsLongLease)
            {
                int prices = 0;
                if (!value.Value.IsRentAction)
                {
                    if ((int)value.Value.RentStartDate.DayOfWeek == 5 && (int)value.Value.RentStartDate.Hour == 16 && (int)value.Value.RentEndDate.DayOfWeek == 1 && (int)value.Value.RentEndDate.Hour == 9)
                    {
                        prices = _carRepository.GetPriceCarForWeek(value.Value.CarID);
                    }
                    else
                    {
                        prices = _carRepository.GetPriceCarForDays(value.Value.CarID, (int)(value.Value.RentEndDate - value.Value.RentStartDate).TotalDays);
                    }
                }
                else
                {

                }
                var entity = new Models.Entities.RentalOrder
                {
                    ID = value.Value.ID,
                    DateProcessed = value.Value.DateProcessed,
                    Customer = value.Value.Customer,
                    Address = value.Value.Address,
                    City = value.Value.City,
                    ZIPCode = value.Value.ZIPCode,
                    Phone = value.Value.Phone,
                    Email = value.Value.Email,
                    RentStartDate = value.Value.RentStartDate,
                    RentEndDate = value.Value.RentEndDate,
                    ParkingPlace = value.Value.ParkingPlace,
                    Days = (int)(value.Value.RentEndDate - value.Value.RentStartDate).TotalDays,
                    OrderTotal = prices,
                    OrderStatus = value.Value.OrderStatus,
                    IsStorno = value.Value.IsStorno,
                    IsReservation = value.Value.IsReservation,
                    IsRent = value.Value.IsRent,
                    IsReturn = value.Value.IsReturn,
                    IsLongLease = false,
                    IsRentAction = value.Value.IsRentAction,
                    RentActionID = value.Value.RentActionID,
                    LimitLongLease = false,
                    AccessoriesID = value.Value.AccessoriesID,
                    Notes = value.Value.Notes,
                    StartMilleage = value.Value.StartMilleage,
                    EndMilleage = value.Value.EndMilleage,
                    Updated = DateTime.Now
                };
                db.RentalOrders.Attach(entity);
                db.Entry(entity).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                db.SaveChanges();
            }
            else
            {
                if (value.Value.LimitLongLease)
                {
                    int prices = 0;
                    var months = GetMonthDifference(value.Value.RentStartDate, value.Value.RentEndDate.AddMonths(-13));
                    if (!value.Value.IsRentAction)
                    {
                        prices = _carRepository.GetLongPriceForCar(value.Value.CarID, months);
                    }
                    else
                    {

                    }
                    var entity = new Models.Entities.RentalOrder
                    {
                        ID = value.Value.ID,
                        DateProcessed = value.Value.DateProcessed,
                        Customer = value.Value.Customer,
                        Address = value.Value.Address,
                        City = value.Value.City,
                        ZIPCode = value.Value.ZIPCode,
                        Phone = value.Value.Phone,
                        Email = value.Value.Email,
                        RentStartDate = value.Value.RentStartDate,
                        RentEndDate = value.Value.RentEndDate,
                        ParkingPlace = value.Value.ParkingPlace,
                        Days = (int)(value.Value.RentEndDate - value.Value.RentStartDate).TotalDays,
                        OrderTotal = prices,
                        OrderStatus = value.Value.OrderStatus,
                        IsStorno = value.Value.IsStorno,
                        IsReservation = value.Value.IsReservation,
                        IsRent = value.Value.IsRent,
                        IsReturn = value.Value.IsReturn,
                        IsLongLease = value.Value.IsLongLease,
                        IsRentAction = value.Value.IsRentAction,
                        RentActionID = value.Value.RentActionID,
                        LimitLongLease = true,
                        AccessoriesID = value.Value.AccessoriesID,
                        Notes = value.Value.Notes,
                        StartMilleage = value.Value.StartMilleage,
                        EndMilleage = value.Value.EndMilleage,
                        Updated = DateTime.Now
                    };
                    db.RentalOrders.Attach(entity);
                    db.Entry(entity).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                    db.SaveChanges();
                }
                else
                {
                    int prices = 0;
                    var months = GetMonthDifference(value.Value.RentStartDate, value.Value.RentEndDate.AddMonths(-13));
                    prices = _carRepository.GetLongNoLimitPriceForCar(value.Value.CarID, months);
                    var entity = new Models.Entities.RentalOrder
                    {
                        ID = value.Value.ID,
                        DateProcessed = value.Value.DateProcessed,
                        Customer = value.Value.Customer,
                        Address = value.Value.Address,
                        City = value.Value.City,
                        ZIPCode = value.Value.ZIPCode,
                        Phone = value.Value.Phone,
                        Email = value.Value.Email,
                        RentStartDate = value.Value.RentStartDate,
                        RentEndDate = value.Value.RentEndDate,
                        ParkingPlace = value.Value.ParkingPlace,
                        Days = (int)(value.Value.RentEndDate - value.Value.RentStartDate).TotalDays,
                        OrderTotal = prices,
                        OrderStatus = value.Value.OrderStatus,
                        IsStorno = value.Value.IsStorno,
                        IsReservation = value.Value.IsReservation,
                        IsRent = value.Value.IsRent,
                        IsReturn = value.Value.IsReturn,
                        IsLongLease = value.Value.IsLongLease,
                        IsRentAction = value.Value.IsRentAction,
                        RentActionID = value.Value.RentActionID,
                        LimitLongLease = false,
                        AccessoriesID = value.Value.AccessoriesID,
                        Notes = value.Value.Notes,
                        StartMilleage = value.Value.StartMilleage,
                        EndMilleage = value.Value.EndMilleage,
                        Updated = DateTime.Now
                    };
                    db.RentalOrders.Attach(entity);
                    db.Entry(entity).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                    db.SaveChanges();
                }
            }
            if (value.Value.IsStorno)
            {
                var rentEventId = db.CarOrderRentEvents.Where(x => x.RentalOrderID == value.Value.ID).Select(x => x.RentEventID).FirstOrDefault();
                var rentEvent = db.RentEvents.Where(x => x.ID == rentEventId).FirstOrDefault();
                db.RentEvents.Remove(rentEvent);
                db.SaveChanges();
                var updateOrder = new Models.Entities.RentalOrder
                {
                    ID = value.Value.ID,
                    OrderStatus = "Zrušeno"
                };
                db.RentalOrders.Attach(updateOrder);
                db.Entry(updateOrder).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                db.SaveChanges();
            }
            else if (value.Value.IsRent)
            {
                var updateOrder = new Models.Entities.RentalOrder
                {
                    ID = value.Value.ID,
                    OrderStatus = "Půjčeno"
                };
                db.RentalOrders.Attach(updateOrder);
                db.Entry(updateOrder).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                db.SaveChanges();

                var rentEventId = db.CarOrderRentEvents.Where(x => x.RentalOrderID == value.Value.ID).Select(x => x.RentEventID).FirstOrDefault();

                var rentEvent = new Models.Entities.RentEvent
                {
                    ID = rentEventId,
                    StartTime = value.Value.RentStartDate,
                    EndTime = value.Value.RentEndDate,
                    Subject = _carRepository.GetCarById(value.Value.CarID).TagNumber + " - " + _carRepository.GetCarById(value.Value.CarID).Make + " " + _carRepository.GetCarById(value.Value.CarID).Model,
                    Description = "Půjčení vozu " + _carRepository.GetCarById(value.Value.CarID).TagNumber + ", místo vrácení: " + value.Value.ParkingPlace,
                    IsAllDay = false,
                    OwnerTaskID = value.Value.OwnerTaskId
                };
                db.RentEvents.Attach(rentEvent);
                db.Entry(rentEvent).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                db.SaveChanges();
            }
            else if (value.Value.IsReturn)
            {
                var updateOrder = new Models.Entities.RentalOrder
                {
                    ID = value.Value.ID,
                    OrderStatus = "Vráceno",
                    EndMilleage = value.Value.EndMilleage
                };
                db.RentalOrders.Attach(updateOrder);
                db.Entry(updateOrder).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                db.SaveChanges();

                var updateTachometer = new Models.Entities.Car
                {
                    ID = value.Value.CarID,
                    Tachometer = Convert.ToInt32(value.Value.EndMilleage)
                };
                db.Cars.Attach(updateTachometer);
                db.Entry(updateTachometer).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                db.SaveChanges();
            }

            return new JsonResult(value.Value);
        }

        public JsonResult InsertAction([FromBody]CRUDModel<Models.RentActionViewModel> value)
        {
            var entity = new Models.Entities.RentAction
            {
                ValidFrom = value.Value.ValidFrom,
                ValidTo = value.Value.ValidTo,
                IsForRegisteredUser = value.Value.IsForRegisteredUser,
                IsActive = value.Value.IsActive,
                Sale = Convert.ToInt32(value.Value.Sale),
                ActionInWorkDays = value.Value.ActionInWorkDays,
                DayRentStatement = Convert.ToInt32(value.Value.DayRentStatement),
                ActionInWeek = value.Value.ActionInWeek,
                DayOfWeekRentStatement = Convert.ToInt32(value.Value.DayOfWeekRentStatement),
                Inserted = System.DateTime.Now,
                Updated = System.DateTime.Now,
            };

            db.RentActions.Add(entity);
            db.SaveChanges();

            int lastActionId = db.RentActions.Select(x => x.ID).Last();

            List<int> IDs = value.Value.CarsInActionId.Split(",").Select(s => int.Parse(s)).ToList();

            int carInActionCount = IDs.Count();
            for (var i = 0; i < carInActionCount + 1; i++)
            {
                var carInAction = new Models.Entities.CarInAction
                {
                    CarID = IDs[i],
                    ActionID = lastActionId
                };
                db.CarInActions.Add(carInAction);
                db.SaveChanges();
            }

            return new JsonResult(value.Value);
        }

        public JsonResult UpdateAction([FromBody]CRUDModel<Models.RentActionViewModel> value)
        {
            var entity = new Models.Entities.RentAction
            {
                ID = value.Value.ID,
                ValidFrom = value.Value.ValidFrom,
                ValidTo = value.Value.ValidTo,
                IsForRegisteredUser = value.Value.IsForRegisteredUser,
                IsActive = true,
                Sale = Convert.ToInt32(value.Value.Sale * 100),
                ActionInWorkDays = value.Value.ActionInWorkDays,
                DayRentStatement = Convert.ToInt32(value.Value.DayRentStatement),
                ActionInWeek = value.Value.ActionInWeek,
                DayOfWeekRentStatement = Convert.ToInt32(value.Value.DayOfWeekRentStatement),
                Updated = System.DateTime.Now
            };
            db.RentActions.Attach(entity);
            db.Entry(entity).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
            //db.SaveChanges();

            return new JsonResult(value.Value);
        }
        
        public class MonthViewModel
        {
            public int value;
            public string text;
        }
        public class Data
        {
            public bool requiresCounts { get; set; }
            public int skip { get; set; }
            public int take { get; set; }
        }
        
        public static int GetMonthDifference(DateTime start, DateTime end)
        {
            int monthsApart = 12 * (start.Year - end.Year) + start.Month - end.Month;
            return Math.Abs(monthsApart);
        }

        public string GetEmplyeeEmailByUserId(string userId)
        {
            string employeeDataSource;
            if (!String.IsNullOrEmpty(userId))
            {
                var lister = new ListUsers(context);
                var data = lister.ListUserWithRoles();
                employeeDataSource = data.Where(u => u.UserID == userId).Select(u => u.Email).FirstOrDefault();
            }
            else
            {
                employeeDataSource = "Nepřidělena";
            }
            return employeeDataSource;
        }
        #endregion
    }
}